"""Convert rowdata (json in rg-grid) to docx."""
import os
import sys

from loguru import logger
from set_loglevel import set_loglevel

# if LOGURU_LEVEL is set use it
# otherwise set according to set_loglevel, default loglevel "INFO"/20
if os.environ.get("LOGURU_LEVEL") is None:
    logger.remove()
    logger.add(sys.stderr, level=set_loglevel())


def rowdata2file():
    """Define rowdata2file."""
    logger.debug(" entry ")



"""Convert rowdata (json in rg-grid) to docx."""
# pylint: disable=invalid-name, no-member
import os
import sys
from pathlib import Path
from typing import Optional, Union

from docx import Document
from docx.enum.text import WD_COLOR_INDEX
from docx.shared import RGBColor
from loguru import logger
from set_loglevel import set_loglevel

# if LOGURU_LEVEL is set use it
# otherwise set according to set_loglevel, default loglevel "INFO"/20
if os.environ.get("LOGURU_LEVEL") is None:
    logger.remove()
    logger.add(sys.stderr, level=set_loglevel())
pdir = Path(__file__).parent


def rowdata2docx(rowdata: dict, infilepath: Optional[Union[str, Path]]) -> str:
    """Convert rowdata (json in rg-grid) to docx.

    Arguments:
        rowdata: [{"text1": ..., "text2": ...},  {"text1": ..., "text2": ...}, ]
        infilepath: input filepath, outfilepath = Path(infilepath).with_suffix('docx')

    Returns:
        message f"saved to {outfilepath}"

    """
    outfilepath = Path(infilepath).with_suffix(".docx")

    _ = Path(pdir, "templ_dual.docx")
    if _.exists():
        document = Document(_)
        logger.info(f"Using template {_}")
    else:
        logger.info(" no template found ")
        logger.info("(A template file dictates fonts, line spacing, margins, etc.)")
        document = Document()

    # def add_para(document, elm: str, color: bool = False, highlight: bool = True):
    def add_para(document, elm: str, color: bool = False, highlight_color=None):
        paragraph = document.add_paragraph()

        # remove leading and trailing spaces
        try:
            elm = elm.strip()
        except Exception as exc:
            logger.error(exc)

        run = paragraph.add_run(elm)
        font = run.font
        if highlight_color:
            # font.highlight_color = WD_COLOR_INDEX.GRAY_25  # pylint: disable=E1101
            # font.highlight_color = WD_COLOR_INDEX.WHITE  # pylint: disable=E1101  8
            font.highlight_color = highlight_color  # pylint: disable=E1101  8

            # https://www.rapidtables.com/web/color/Yellow_Color.html
            # lightyellow	#FFFFE0	rgb(255,255,224)
            # yellow	#FFFF00	rgb(255,255,0)
            # font.highlight_color = RGBColor(0xFF, 0xFF, 0xE0)  # does no work pylint: disable=E1101
        if color:
            font.color.rgb = RGBColor(0xFF, 0xFF, 0xE0)  # noqa

    for elm in rowdata:
        add_para(document, elm["text1"], False, WD_COLOR_INDEX.YELLOW)
        add_para(document, elm["text2"], False, False)

        if elm.get("text3"):  # WD_COLOR_INDEX.GRAY_25
            add_para(document, elm["text1"], False, WD_COLOR_INDEX.GRAY_25)

    try:
        document.save(outfilepath)
        _ = f"\n\t Saved to {outfilepath}"
        logger.info(_)

        print(_)  # for python-shell process

        # write-permission problem: would this help?
        # del document
    except Exception as exc:
        _ = f" Failed to save {outfilepath}, exc: {exc}"
        logger.error(_)
        print(_)

    return _



"""Prep __main__.py."""
# pylint: disable=invalid-name
from pathlib import Path
from typing import Optional

import logzero
import typer
from logzero import logger
from set_loglevel import set_loglevel

from rowdata2file import __version__, rowdata2file

del Path, logger, rowdata2file
logzero.loglevel(set_loglevel())

app = typer.Typer(
    name="rowdata2file",
    add_completion=False,
    help="rowdata2file help",
)


def _version_callback(value: bool) -> None:
    if value:
        typer.echo(f"{app.info.name} v.{__version__} -- ...")
        raise typer.Exit()


@app.command()
def main(
    version: Optional[bool] = typer.Option(  # pylint: disable=(unused-argument
        None,
        "--version",
        "-v",
        "-V",
        help="Show version info and exit.",
        callback=_version_callback,
        is_eager=True,
    ),
):
    """Define."""


if __name__ == "__main__":
    app()



"""Convert rowdata (json in rg-grid) to docx table(side by side)."""
# pylint: disable=invalid-name, no-member
import os
import sys
from pathlib import Path
from typing import Optional, Union

from docx import Document
from loguru import logger
from set_loglevel import set_loglevel

# if LOGURU_LEVEL is set use it
# otherwise set according to set_loglevel, default loglevel "INFO"/20
if os.environ.get("LOGURU_LEVEL") is None:
    logger.remove()
    logger.add(sys.stderr, level=set_loglevel())
pdir = Path(__file__).parent


def rowdata2docx_t(rowdata: dict, infilepath: Optional[Union[str, Path]]) -> str:
    """Convert rowdata (json in rg-grid) to docx table (side by side).

    Arguments:
        rowdata: [{"text1": ..., "text2": ...},  {"text1": ..., "text2": ...}, ]
        infilepath: input filepath, outfilepath = Path(infilepath).with_suffix('docx')

    Returns:
        save a doxc table (side by side)
        message f"saved to {outfilepath}"
    """
    # outfilepath = Path(infilepath).with_suffix(".docx")
    stem = Path(infilepath).stem
    outfilepath = infilepath.with_name(f"{stem}-t.docx")

    _ = Path(pdir, "templ_dual.docx")
    if _.exists():
        document = Document(_)
        logger.info(f"Using template {_}")
    else:
        logger.info(" no template found ")
        logger.info("(A template file dictates fonts, line spacing, margins, etc.)")
        document = Document()

    rows = len(rowdata)
    if not rows:
        logger.warning("No data in rowdata...")
        return "No data..."

    cols = len(rowdata[0])
    keys = [*rowdata[0]]  # ['text1', 'text2', 'text3']

    # chatgpt python code to generate docx table
    table = document.add_table(rows=rows, cols=cols)
    for i in range(rows):
        row_cells = table.rows[i].cells
        for j in range(cols):
            # row_cells[0].text = f'Row {i}, Cell 1'
            # row_cells[1].text = rowdata[i].get(keys[j], "")
            row_cells[j].text = rowdata[i][keys[j]]

    try:
        document.save(outfilepath)
        _ = f"\n\t Saved to {outfilepath}"
        logger.info(_)

        print(_)  # for python-shell process

        # write-permission problem: would this help?
        # del document
    except Exception as exc:
        _ = f" Failed to save {outfilepath}, exc: {exc}"
        logger.error(_)
        print(_)

    return _



"""Generate docx from two lists/texts."""
# pylint: disable=duplicate-code

from itertools import zip_longest
from pathlib import Path
from typing import List, Union

from docx import Document
from docx.enum.text import WD_COLOR_INDEX

# from docx.shared import Pt
# from docx.oxml.ns import qn
from docx.shared import RGBColor
from logzero import logger

# from textwrap import shorten


# fmt: off
def gen_docx(
        src: Union[str, List[str]],
        tgt: Union[str, List[str]],
        template: str = "templ_dual.docx",
) -> Document:
    # fmt: on
    """Generate docx from two lists/texts.

    tempplate default to templ_dual.docx in the current directory if pesent
    """
    if isinstance(src, str):
        src = [elm.strip() for elm in src.splitlines() if elm.strip()]
    if isinstance(tgt, str):
        tgt = [elm.strip() for elm in tgt.splitlines() if elm.strip()]

    if not len(src) == len(tgt):
        logger.warning(" lent(src) %s and len(tgt) %s not match", len(src), len(tgt))
        logger.warning("There appears to be some problem.")
        logger.warning("We proceed nevertheless.")

    _ = Path(template).expanduser().resolve()
    if _.exists():
        document = Document(_)
        logger.info("Using template %s", template)
    else:
        logger.info(" %s not present, no template file used", template)
        logger.info("(A template file dictates fonts, line spacing, margins, etc.)")
        document = Document()

    # def add_para(document, elm: str, color: bool = True, highlight: bool = False):
    def add_para(document, elm: str, color: bool = False, highlight: bool = True):
        paragraph = document.add_paragraph()

        # remove leading and trailing spaces
        try:
            elm = elm.strip()
        except Exception as exc:
            logger.error(exc)

        run = paragraph.add_run(elm)
        font = run.font
        if highlight:
            # font.highlight_color = WD_COLOR_INDEX.GRAY_25  # pylint: disable=E1101
            # font.highlight_color = WD_COLOR_INDEX.WHITE  # pylint: disable=E1101  8
            font.highlight_color = WD_COLOR_INDEX.YELLOW  # pylint: disable=E1101  8

            # https://www.rapidtables.com/web/color/Yellow_Color.html
            # lightyellow	#FFFFE0	rgb(255,255,224)
            # yellow	#FFFF00	rgb(255,255,0)
            # font.highlight_color = RGBColor(0xFF, 0xFF, 0xE0)  # does no work pylint: disable=E1101
        if color:
            font.color.rgb = RGBColor(0xff, 0xff, 0xe0)  # noqa

    for elm0, elm1 in zip_longest(src, tgt, fillvalue=""):
        add_para(document, elm0, False, True)  #
        add_para(document, elm1, False, False)  #

    return document



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rowdata2file']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.6.0,<0.7.0',
 'logzero>=1.7.0,<2.0.0',
 'python-docx>=0.8.11,<0.9.0',
 'rich>=13.3.1,<14.0.0',
 'set-loglevel>=0.1.2,<0.2.0',
 'typer>=0.7.0,<0.8.0']

entry_points = \
{'console_scripts': ['rowdata2file = rowdata2file.__main__:app']}

setup_kwargs = {
    'name': 'rowdata2file',
    'version': '0.1.1',
    'description': 'Convert rowdata (ag-grid) to a file for python-shell use',
    'long_description': '# rowdata2file\n[![pytest](https://github.com/ffreemt/rowdata2file/actions/workflows/routine-tests.yml/badge.svg)](https://github.com/ffreemt/rowdata2file/actions)[![python](https://img.shields.io/static/v1?label=python+&message=3.8%2B&color=blue)](https://www.python.org/downloads/)[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)[![PyPI version](https://badge.fury.io/py/rowdata2file.svg)](https://badge.fury.io/py/rowdata2file)\n\nConvert rowdata (ag-grid) to a file for python-shell use\n\n## Install it\n\n```shell\npip install rowdata2file\n# pip install git+https://github.com/ffreemt/rowdata2file\n# poetry add git+https://github.com/ffreemt/rowdata2file\n# git clone https://github.com/ffreemt/rowdata2file && cd rowdata2file\n```\n\n## Use it\n```python\nfrom rowdata2file import rowdata2docx, rowdata2docx_t\n\n```\n',
    'author': 'ffreemt',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ffreemt/rowdata2file',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.3,<4.0.0',
}


setup(**setup_kwargs)
